module.exports = function(RED) {
  "use strict";
  const { StorageProvider } = require('nodejs-helpers');

  function GoogleStorageOutNode(n) {
    RED.nodes.createNode(this, n);
    this.topic = n.topic;
    this.bucketName = n.bucketName;
    this.forBigQuery = n.forBigQuery;
    this.fileName = n.fileName;
    const node = this;

    const storage = new StorageProvider({
      bucketName: node.bucketName,
      forBigQuery: node.forBigQuery === "true"
    });

    node.on("input", function(msg) {
      if (msg.hasOwnProperty("payload")) {
        const filename = `${msg.fileName || node.fileName}`.replace(/\{timestamp\}/, `${Date.now()}`);

        node.status({ fill: "green", shape: "ring", text: "saving" });

        if (typeof msg.params === 'undefined') { msg.params = {}; }

        storage.save(filename, msg.payload, msg.params)
          .then(function(data) {
            node.status({ fill: "green", shape: "dot", text: "saved" });
            const msgOut = {
              connectionId: msg.connectionId,
              ackId: msg.ackId,
              received: msg.received,
            };
            node.send(msgOut);
          })
          .catch(function(err) {
            node.status({ fill: "red", shape: "ring", text: "failed" });
            node.error(err, msg);
          });
      } else { node.warn(RED._("google-storage.errors.nopayload")); }
    });

    node.status({ fill: "blue", shape: "dot", text: "ready" });
  }
  RED.nodes.registerType("google-storage out", GoogleStorageOutNode);
}
